/* Header for module bx_dev, generated by p2c */
#ifndef _DV_ALL_HEADER_
#define _DV_ALL_HEADER_

#include "bx_types.h"
#include "bx_read.h"

typedef struct {
  char  *dev;
  void (*Ask)(void);
  void (*Init)(double *xpos, double *ypos);
  void (*Setcolor)(int colno);
  void (*Charout)(char c, double *xpos, double *ypos);
  void (*Stringout)(char *s,  double *xpos, double *ypos);
  void (*Newline)(double *xpos, double *ypos);
  void (*Newpage)(double *xpos, double *ypos);
  void (*Exit)(void);
} GraphicsDevice;

extern GraphicsDevice Ansi;
extern GraphicsDevice Ascii;
#ifdef oCRT
extern GraphicsDevice Crt;
#endif
extern GraphicsDevice Rtf;
extern GraphicsDevice Pict;
extern GraphicsDevice Postscript;
extern GraphicsDevice Eps;
extern GraphicsDevice Hpgl;
extern GraphicsDevice Lj250;
extern GraphicsDevice Fig;
extern GraphicsDevice RegisT, RegisP;
extern GraphicsDevice Vt;
extern GraphicsDevice Html;

extern void GenericStringOut(char *s,  double *xpos, double *ypos);

#endif /* _DV_ALL_HEADER_ */

